from datetime import datetime
import traceback
from functools import wraps

# 异常输出
def except_output(msg='异常'):
    # msg用于自定义函数的提示信息
    def except_execute(func):
        @wraps(func)
        def execept_print(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                sign = '=' * 60 + '\n'
                print(f'{sign}>>>异常时间：\t{datetime.now()}\n>>>异常函数：\t{func.__name__}\n>>>{msg}：\t{e}')
                print(f'{sign}{traceback.format_exc()}{sign}')
        return execept_print
    return except_execute


from externals.libhdmap import *
from pyhdmap.utils.singleton import Singleton
from pyhdmap.remote.grpc_client import HDMapClient
from pyhdmap.remote.hdmap_impl.geometry import LineStringDataImpl
from pyhdmap.remote.hdmap_impl.lane_impl import LaneImpl
from pyhdmap.remote.hdmap_impl.road_impl import RoadImpl
from pyhdmap.remote.hdmap_impl.junction_impl import JunctionImpl
from pyhdmap.remote.hdmap_impl.object_impl import ObjectImpl
from pyhdmap.remote.hdmap_impl.poi_impl import PoiImpl
from pyhdmap.remote.hdmap_impl.parking_space_impl import ParkingSpaceImpl
from pyhdmap.remote.hdmap_impl.pull_over_region_impl import PullOverRegionImpl
from pyhdmap.remote.hdmap_impl.stop_line_impl import StopLineImpl
from pyhdmap.remote.hdmap_impl.crosswalk_impl import CrosswalkImpl
from pyhdmap.remote.hdmap_impl.signal_impl import SignalImpl
from pyhdmap.remote.builder import Builder

class MapInfo:
  def __init__(self) -> None:
    self._line_list = []
    self._lane_list = []
    self._road_list = []
    self._junction_list = []
    self._object_list = []
    self._poi_list = []
    self._parking_space_list = []
    self._pull_over_region_list = []
    self._stop_line_list = []
    self._crosswalk_list = []
    self._signal_list = []
  
  def lines(self):
    return self._line_list
  
  def lanes(self):
    return self._lane_list
  
  def roads(self):
    return self._road_list
  
  def junctions(self):
    return self._junction_list
  
  def objects(self):
    return self._object_list
  
  def pois(self):
    return self._poi_list
  
  def parking_spaces(self):
    return self._parking_space_list
  
  def pull_over_regions(self):
    return self._pull_over_region_list
  
  def stop_lines(self):
    return self._stop_line_list
  
  def crosswalks(self):
    return self._crosswalk_list
  
  def signals(self):
    return self._signal_list
    

@Singleton
class ImplCache:
  def __init__(self) -> None:
    self._line_impl_cache:dict[int, LineStringDataImpl] = {}
    self._lane_impl_cache:dict[int, LaneImpl] = {}
    self._road_impl_cache:dict[int, RoadImpl] = {}
    self._junction_impl_cache:dict[int, JunctionImpl] = {}
    self._object_impl_cache:dict[int, ObjectImpl] = {}
    self._poi_impl_cache:dict[int, PoiImpl] = {}
    self._parking_space_impl_cache:dict[int, ParkingSpaceImpl] = {}
    self._pull_over_region_impl_cache:dict[int, PullOverRegionImpl] = {}
    self._stop_line_impl_cache:dict[int, StopLineImpl] = {}
    self._crosswalk_impl_cache:dict[int, CrosswalkImpl] = {}
    self._signal_impl_cache:dict[int, SignalImpl] = {}
    
    self._map_info = MapInfo()
    self._map_info_generated = False
    
    self.hdmap_client = HDMapClient()
    print("impl cache init finish")
    
  def set_config(self, ip, port, scene):
    self.hdmap_client.set_business_scene(scene)
    self.hdmap_client.set_ip_and_port(ip, port)
    
  def build_impls_from_getmap_response(self, grpc_response):
    for lane_proto in grpc_response.lane_proto:
      if lane_proto.id.id in self._lane_impl_cache:
        continue
      impl, line_impls = Builder.build_lane_impl(lane_proto, grpc_response.lines, grpc_response.relations, grpc_response.lane_links)
      self._lane_impl_cache[impl.id()] = impl
      self._map_info._lane_list.append(impl)
      for line_impl in line_impls:
        self._line_impl_cache[line_impl.id()] = line_impl
        self._map_info._line_list.append(line_impl)
    for road_proto in grpc_response.road_proto:
      if road_proto.id.id in self._road_impl_cache:
        continue
      impl = Builder.build_road_impl(road_proto, grpc_response.lines)
      self._road_impl_cache[impl.id()] = impl
      self._map_info._road_list.append(impl)
    for junction_proto in grpc_response.junction_proto:
      if junction_proto.id.id in self._junction_impl_cache:
        continue
      impl = Builder.build_junction_impl(junction_proto, grpc_response.lines, grpc_response.relations)
      self._junction_impl_cache[impl.id()] = impl
      self._map_info._junction_list.append(impl)
    for object_proto in grpc_response.object_proto:
      if object_proto.id.id in self._object_impl_cache:
        continue
      impl = Builder.build_object_impl(object_proto, grpc_response.lines, grpc_response.relations)
      self._object_impl_cache[impl.id()] = impl
      self._map_info._object_list.append(impl)
    for crosswalk_proto in grpc_response.crosswalk_proto:
      if crosswalk_proto.id.id in self._crosswalk_impl_cache:
        continue
      impl, line_impls = Builder.build_crosswalk_impl(crosswalk_proto, grpc_response.lines, grpc_response.relations)
      self._crosswalk_impl_cache[impl.id()] = impl
      self._map_info._crosswalk_list.append(impl)
      for line_impl in line_impls:
        self._line_impl_cache[line_impl.id()] = line_impl
    for parking_space_proto in grpc_response.parking_space_proto:
      if parking_space_proto.id.id in self._parking_space_impl_cache:
        continue
      impl, line_impls = Builder.build_parking_space_impl(parking_space_proto, grpc_response.lines, grpc_response.relations)
      self._parking_space_impl_cache[impl.id()] = impl
      self._map_info._parking_space_list.append(impl)
      for line_impl in line_impls:
        self._line_impl_cache[line_impl.id()] = line_impl
    for poi_proto in grpc_response.poi_proto:
      if poi_proto.id.id in self._poi_impl_cache:
        continue
      impl = Builder.build_poi_impl(poi_proto, grpc_response.relations)
      self._poi_impl_cache[impl.id()] = impl
      self._map_info._poi_list.append(impl)
    for pull_over_region_proto in grpc_response.pull_over_region_proto:
      if pull_over_region_proto.id.id in self._pull_over_region_impl_cache:
        continue
      impl = Builder.build_pull_over_region_impl(pull_over_region_proto, grpc_response.relations)
      self._pull_over_region_impl_cache[impl.id()] = impl
      self._map_info._pull_over_region_list.append(impl)
    for signal_proto in grpc_response.signal_proto:
      if signal_proto.id.id in self._signal_impl_cache:
        continue
      impl = Builder.build_signal_impl(signal_proto, grpc_response.lines, grpc_response.relations)
      self._signal_impl_cache[impl.id()] = impl
      self._map_info._signal_list.append(impl)
    for stop_line_proto in grpc_response.stop_line_proto:
      if stop_line_proto.id.id in self._stop_line_impl_cache:
        continue
      impl, line_impls = Builder.build_stop_line_impl(stop_line_proto, grpc_response.lines, grpc_response.relations)
      self._stop_line_impl_cache[impl.id()] = impl
      self._map_info._stop_line_list.append(impl)
      for line_impl in line_impls:
        self._line_impl_cache[line_impl.id()] = line_impl
    self._map_info_generated = True
    return 
        
  def build_impls_from_proto(self, grpc_response, type):
    impls = []
    total_line_impls = []
    if type == "lane":
      for element in grpc_response.element_proto:
        impl, line_impls = Builder.build_lane_impl(element.lane_proto, grpc_response.lines, grpc_response.relations, grpc_response.lane_links)
        impls.append(impl)
        total_line_impls.append(line_impls)
    elif type == "road":
      for element in grpc_response.element_proto:
        impl = Builder.build_road_impl(element.road_proto, grpc_response.lines)
        impls.append(impl)
    elif type == "junction":
      for element in grpc_response.element_proto:
        impl = Builder.build_junction_impl(element.junction_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
    elif type == "object":
      for element in grpc_response.element_proto:
        impl = Builder.build_object_impl(element.object_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
    elif type == "crosswalk":
      for element in grpc_response.element_proto:
        impl, line_impls = Builder.build_crosswalk_impl(element.crosswalk_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
        total_line_impls.append(line_impls)
    elif type == "parking_space":
      for element in grpc_response.element_proto:
        impl, line_impls = Builder.build_parking_space_impl(element.parking_space_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
        total_line_impls.append(line_impls)
    elif type == "poi":
      for element in grpc_response.element_proto:
        impl = Builder.build_poi_impl(element.poi_proto, grpc_response.relations)
        impls.append(impl)
    elif type == "pull_over_region":
      for element in grpc_response.element_proto:
        impl = Builder.build_pull_over_region_impl(element.pull_over_region_proto, grpc_response.relations)
        impls.append(impl)
    elif type == "signal":
      for element in grpc_response.element_proto:
        impl = Builder.build_signal_impl(element.signal_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
    elif type == "stop_line":
      for element in grpc_response.element_proto:
        impl, line_impls = Builder.build_stop_line_impl(element.stop_line_proto, grpc_response.lines, grpc_response.relations)
        impls.append(impl)
        total_line_impls.append(line_impls)
        
    return impls, total_line_impls
      
    
  def get_impl_by_ids(self, type, ids):
    if type == "lane":
      impl_cache = self._lane_impl_cache
    elif type == "line":  
      impl_cache = self._line_impl_cache
    elif type == "road":
      impl_cache = self._road_impl_cache
    elif type == "junction":
      impl_cache = self._junction_impl_cache
    elif type == "object":
      impl_cache = self._object_impl_cache
    elif type == "crosswalk":
      impl_cache = self._crosswalk_impl_cache
    elif type == "parking_space":
      impl_cache = self._parking_space_impl_cache
    elif type == "poi":
      impl_cache = self._poi_impl_cache
    elif type == "pull_over_region":
      impl_cache = self._pull_over_region_impl_cache
    elif type == "signal":
      impl_cache = self._signal_impl_cache
    elif type == "stop_line":
      impl_cache = self._stop_line_impl_cache
    
    res = []
    query_ids = []
    if isinstance(ids, list):
      for id in ids:
        if id in impl_cache.keys():
          res.append(impl_cache[id])
        else:
          query_ids.append(id)
    else:
      if ids in impl_cache.keys():
        res.append(impl_cache[ids])
      else:
        query_ids.append(ids)
        
    if len(query_ids) == 0:
      return res
    
    response = self.hdmap_client.get_elements_by_ids(query_ids, type)
    if response == None:
      pass
    
    impls, total_line_impls = self.build_impls_from_proto(response, type)

    if len(total_line_impls) != 0:
      for line_impl in total_line_impls:
        self._line_impl_cache[line_impl.id()] = line_impl

    for impl in impls:
      impl_cache[impl.id()] = impl 
    res.append(impls)
    return res
  
if __name__ == "__main__":
  impl_cache = ImplCache()
  print(impl_cache.get_impl_by_id("lane", 1))
    
  


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/hdmap_lib/proto/poi.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from proto.hdmap_lib.proto import id_pb2 as proto_dot_hdmap__lib_dot_proto_dot_id__pb2
from proto.hdmap_lib.proto import geometry_pb2 as proto_dot_hdmap__lib_dot_proto_dot_geometry__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/hdmap_lib/proto/poi.proto',
  package='hdmap_lib',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1fproto/hdmap_lib/proto/poi.proto\x12\thdmap_lib\x1a\x1eproto/hdmap_lib/proto/id.proto\x1a$proto/hdmap_lib/proto/geometry.proto\"\xa4\x02\n\x03Poi\x12\x19\n\x02id\x18\x01 \x01(\x0b\x32\r.hdmap_lib.Id\x12\x0c\n\x04name\x18\x02 \x01(\t\x12$\n\x04type\x18\x03 \x01(\x0e\x32\x16.hdmap_lib.Poi.PoiType\x12&\n\ncoordinate\x18\x04 \x01(\x0b\x32\x12.hdmap_lib.Point3D\"\xa5\x01\n\x07PoiType\x12\x14\n\x10POI_QUEUE_REGION\x10\x01\x12\x13\n\x0fPOI_REST_REGION\x10\x02\x12\x15\n\x11POI_CHARGE_REGION\x10\x03\x12\x13\n\x0fPOI_LOCK_REGION\x10\x04\x12\x15\n\x11POI_UNLOCK_REGION\x10\x05\x12\x14\n\x10POI_LEAVE_REGION\x10\x06\x12\x16\n\x12POI_OFFLINE_REGION\x10\x07'
  ,
  dependencies=[proto_dot_hdmap__lib_dot_proto_dot_id__pb2.DESCRIPTOR,proto_dot_hdmap__lib_dot_proto_dot_geometry__pb2.DESCRIPTOR,])



_POI_POITYPE = _descriptor.EnumDescriptor(
  name='PoiType',
  full_name='hdmap_lib.Poi.PoiType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='POI_QUEUE_REGION', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_REST_REGION', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_CHARGE_REGION', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_LOCK_REGION', index=3, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_UNLOCK_REGION', index=4, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_LEAVE_REGION', index=5, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='POI_OFFLINE_REGION', index=6, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=244,
  serialized_end=409,
)
_sym_db.RegisterEnumDescriptor(_POI_POITYPE)


_POI = _descriptor.Descriptor(
  name='Poi',
  full_name='hdmap_lib.Poi',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='hdmap_lib.Poi.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='hdmap_lib.Poi.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='hdmap_lib.Poi.type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='coordinate', full_name='hdmap_lib.Poi.coordinate', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _POI_POITYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=117,
  serialized_end=409,
)

_POI.fields_by_name['id'].message_type = proto_dot_hdmap__lib_dot_proto_dot_id__pb2._ID
_POI.fields_by_name['type'].enum_type = _POI_POITYPE
_POI.fields_by_name['coordinate'].message_type = proto_dot_hdmap__lib_dot_proto_dot_geometry__pb2._POINT3D
_POI_POITYPE.containing_type = _POI
DESCRIPTOR.message_types_by_name['Poi'] = _POI
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Poi = _reflection.GeneratedProtocolMessageType('Poi', (_message.Message,), {
  'DESCRIPTOR' : _POI,
  '__module__' : 'proto.hdmap_lib.proto.poi_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Poi)
  })
_sym_db.RegisterMessage(Poi)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/hdmap_lib/proto/header.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/hdmap_lib/proto/header.proto',
  package='hdmap_lib',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\"proto/hdmap_lib/proto/header.proto\x12\thdmap_lib\"\x1a\n\nProjection\x12\x0c\n\x04proj\x18\x01 \x01(\t\"\xca\x01\n\x06Header\x12\x11\n\trev_major\x18\x01 \x01(\r\x12\x11\n\trev_minor\x18\x02 \x01(\r\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61te\x18\x04 \x01(\t\x12#\n\x04proj\x18\x05 \x01(\x0b\x32\x15.hdmap_lib.Projection\x12\r\n\x05north\x18\x06 \x01(\x01\x12\r\n\x05south\x18\x07 \x01(\x01\x12\x0c\n\x04\x65\x61st\x18\x08 \x01(\x01\x12\x0c\n\x04west\x18\t \x01(\x01\x12\x0e\n\x06vendor\x18\n \x01(\t\x12\x0c\n\x04name\x18\x0b \x01(\t'
)




_PROJECTION = _descriptor.Descriptor(
  name='Projection',
  full_name='hdmap_lib.Projection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='proj', full_name='hdmap_lib.Projection.proj', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=75,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='hdmap_lib.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rev_major', full_name='hdmap_lib.Header.rev_major', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rev_minor', full_name='hdmap_lib.Header.rev_minor', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='hdmap_lib.Header.version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='date', full_name='hdmap_lib.Header.date', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='proj', full_name='hdmap_lib.Header.proj', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='north', full_name='hdmap_lib.Header.north', index=5,
      number=6, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='south', full_name='hdmap_lib.Header.south', index=6,
      number=7, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='east', full_name='hdmap_lib.Header.east', index=7,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='west', full_name='hdmap_lib.Header.west', index=8,
      number=9, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vendor', full_name='hdmap_lib.Header.vendor', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='hdmap_lib.Header.name', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=280,
)

_HEADER.fields_by_name['proj'].message_type = _PROJECTION
DESCRIPTOR.message_types_by_name['Projection'] = _PROJECTION
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Projection = _reflection.GeneratedProtocolMessageType('Projection', (_message.Message,), {
  'DESCRIPTOR' : _PROJECTION,
  '__module__' : 'proto.hdmap_lib.proto.header_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Projection)
  })
_sym_db.RegisterMessage(Projection)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'proto.hdmap_lib.proto.header_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Header)
  })
_sym_db.RegisterMessage(Header)


# @@protoc_insertion_point(module_scope)



import grpc
import time
from typing import List, Tuple

import proto.grpc.hdmapimpl_pb2 as hdmapimpl_pb2
from proto.grpc import hdmapimpl_pb2_grpc
from pyhdmap.utils.singleton import Singleton
from pyhdmap.utils.vessel import *

@Singleton
class HDMapClient:

  def __init__(self):
    self._ip = "[::]"
    self._port = "50051"
    self._scene = ""
    self._uuid = " "
    self.channel = grpc.insecure_channel(f"{self._ip}:{self._port}")
    self.stub = hdmapimpl_pb2_grpc.HdmapImplStub(self.channel)
    print("grpc client init finish")

  def set_ip_and_port(self, ip, port):
    self._ip = ip
    self._port = port
    self.channel = grpc.insecure_channel(f"{self._ip}:{self._port}")
    self.stub = hdmapimpl_pb2_grpc.HdmapImplStub(self.channel)
  
  def set_business_scene(self, scene):
    self._scene = scene 
    
  ##hdmap interface
  
  async def LoadMap(self):
    request = hdmapimpl_pb2.LoadMapRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    response = await self.stub.LoadMap(request)
    if response.load_status == False:
      return False
    else:
      self._uuid = response.uuid
      return True
    
  async def ReleaseMap(self):
    request = hdmapimpl_pb2.ReleaseMapReply()
    request.time_stamp = time.time()
    request.uuid = self._uuid
    response = await self.stub.ReleaseMap(request)
    return response.status
    
  async def Reload(self, vessels):
    request = hdmapimpl_pb2.ReloadRequest()
    request.time_stamp = time.time()
    request.uuid = self._uuid
    vessel_status = build_vessel_status(vessels)
    request.vessel_status.CopyFrom(vessel_status)
    response = await self.stub.Reload(request)
    return response.reload_status
  
  async def GetMap(self):
    request = hdmapimpl_pb2.GetMapRequest()
    request.time_stamp = time.time()
    request.uuid = self._uuid
    response = await self.stub.GetMap(request)
    return response
      
  
  async def generate_get_element_by_id_request(self, ids, type):
    request = hdmapimpl_pb2.GetElementByIdRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.element_ids.extend(ids) 
    request.element_type = type
    return request
    
  async def get_elements_by_ids(self, ids, type):
    request = self.generate_get_element_by_id_request(ids, type)
    response = await self.stub.GetElementById(request)
    if response.status == False:
      pass #output error
    else:
      return response

  async def generate_get_elements_request(self, point, radius, type):
    request = hdmapimpl_pb2.GetElementsRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.x = point.x()
    request.y = point.y()
    request.element_type = type
    request.radius = radius
    return request
  
  async def get_elements(self, point, radius, type):
    request = self.generate_get_elements_request(point, radius, type)
    response = await self.stub.GetElements(request)
    if response.status == False:
      pass
    else:
      return response
    
  ## lane interface
  async def get_lane_projection(self, lane_id, x, y):
    request = hdmapimpl_pb2.GetProjectionRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.x = x
    request.y = y
    response = await self.stub.Getprojection(request)
    if response.status == False:
      pass
    else:
      return response
    
  async def get_lane_point(self, lane_id, s, d):
    request = hdmapimpl_pb2.GetPointRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.s = s
    request.d = d
    response = await self.stub.GetPoint(request)
    if response.status == False:
      pass
    else:
      return response
    
  async def get_lane_heading(self, lane_id, s):
    request = hdmapimpl_pb2.GetHeadingRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.s = s
    response = await self.stub.GetHeading(request)
    if response.status == False:
      pass
    else:
      return response
    
  async def get_lane_width(self, lane_id, s):
    request = hdmapimpl_pb2.GetWidthRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.s = s
    response = await self.stub.GetWidth(request)
    if response.status == False:
      pass
    else:
      return response
    
  async def get_lane_road_width(self, lane_id, s):
    request = hdmapimpl_pb2.GetWidthRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.s = s
    response = await self.stub.GetRoadWidth(request)
    if response.status == False:
      pass
    else:
      return response
    
  async def get_lane_relative_width(self, lane_id, x, y, heading):
    request = hdmapimpl_pb2.GetRelativeWidthRequest()
    request.time_stamp = time.time()
    request.business_scene = self._scene
    request.lane_id = lane_id
    request.x = x
    request.y = y
    request.heading = heading
    response = await self.stub.GetRelativeWidth(request)
    if response.status == False:
      pass
    else:
      return response


from externals.libhdmap import *
from proto.hdmap_lib.proto import geometry_pb2

class LineStringDataImpl:
  def __init__(self) -> None:
    self._id = 0
    self._points:list[Vec2d] = []
    self._type = geometry_pb2.LineStringData.Type.NONE
    self._color = geometry_pb2.LineStringData.Color.WHITE
    
  def id(self):
    return self._id
  
  def points(self):
    return self._points
  
  def type(self):
    return self._type
  
  def color(self):
    return self._color
  
class LineStringImpl:
  def __init__(self) -> None:
    self._id = 0
    self._is_reverse = False
    
  def id(self):
    return self._id
  
  def id_reverse(self):
    return self._is_reverse
  

class CurveImpl:
  def __init__(self) -> None:
    self._lines:list[LineStringImpl] = []
    
  def lines(self):
    return self._lines 